LineSidePrintingApp.controller('LabelsController', function ($scope, LabelsService) {
    var _this = this;
    _this.SelectedGroup = null;


    _this.isApplicationRunning = function () {
        return LabelsService.ApplicationRunning;
    };


    _this.getGroups = function () {
        return LabelsService.Groups;
    };


    _this.selectedGroup = function (group) {
        _this.SelectedGroup = group;
    };


    _this.getLabels = function () {
        return LabelsService.Labels;
    };


    _this.getSelectedGroupLabels = function () {
        if (null !== _this.SelectedGroup) {
            var labels = [];
            for (var i = 0; i < _this.getLabels().length; i++) {
                var label = _this.getLabels()[i];
                var underscorePos = label.indexOf('_');
                if (-1 != underscorePos) {
                    if (label.startsWith(_this.SelectedGroup)) {
                        // remove selected group
                        labels.push(label.substring(underscorePos + 1));
                    }
                }
            }
            // labels.push("Force Error");

            return labels;
        }
    };


    _this.labelClicked = function (label) {
        var dialog = bootbox.dialog({
            closeButton: false,
            title: "Confirm Print",
            message: '<p>Are you sure you want to print \'' + label + '\'?</p>' +
                '<form role="form">' +
                '    <div class="modal-body">' +
                '        <div class="form-group">' +
                '            <label for="print-preview-quantity">Quantity</label>' +
                '            <input type="text" class="form-control" id="print-preview-quantity" placeholder="Enter Quantity" value="1">' +
                '        </div>' +
                '    </div>' +
                '</form>',
            buttons: {
                submit: {
                    label: "Print",
                    className: "btn-primary",
                    callback: function () {
                        var quantity = $('#print-preview-quantity').val();

                        $("#bootbox-submit-btn").hide();
                        $("#bootbox-cancel-btn").hide();

                        JnrWebsocket.postMessage(5004, JSON.stringify({ Command: "print-label", Message: 'print', LabelName: _this.SelectedGroup + '_' + label, Qty: quantity }));
                        // return false;

                        toastr.info("", "Printing " + label + "...", { timeOut: 2000 });
                    }
                },
                cancel: {
                    label: "Cancel",
                    className: "btn-default"
                }
            }
        });
        dialog.init(function () {
            $(".modal-header").css("background-color", "#428bca");
            $(".modal-header").css("color", "#fff");
        });
    };
});


LineSidePrintingApp.service('LabelsService', function ($rootScope) {
    var _this = this;
    _this.ApplicationRunning = true;
    _this.Groups = [];
    _this.Labels = [];


    // load the configured labels
    _this.load = function () {
    };


    // when our jnior websocket connection has successfully logged in
    JnrWebsocket.onLoggedIn = function () {
        _this.ApplicationRunning = true;
        $rootScope.$apply();

        JnrWebsocket.postMessage(5004, JSON.stringify({ command: "get-labels" }));

        _this.load();
    };


    JnrWebsocket.pingStateChanged = function (connectionState) {
        _this.ApplicationRunning = connectionState;
        $rootScope.$apply();
    };


    JnrWebsocket.onReplyMessage = function (replyContentJson) {
        var contentJson = JSON.parse(replyContentJson.Content);
        if ("error" === contentJson.Command) {
            _this.handleErrorResponse(contentJson);

        } else if ("labels-resp" === contentJson.Command) {
            _this.handleLabelResponse(contentJson);

        } else if ("print-label" === contentJson.Command) {
            if (contentJson.Result) {
                toastr.success("", "Printed Label '" + contentJson.LabelName + "' successfully!", { timeOut: 2000 });
            }

        }

        $rootScope.$apply();
    };


    _this.handleErrorResponse = function (contentJson) {
        /**
         * we only want to show the error if the client id is ours or there is not a client id.
         */
        if (!contentJson.ClientId || contentJson.ClientId == JnrWebsocket.ClientId) {
            var timestamp = new Date().getTime();
            var html = "<a href='javascript: void(0)' data-toggle=\"collapse\" data-target=\"#stacktrace-" + timestamp + "\">Show Stacktrace</a>" +
                "<pre class='stacktrace collapse' id='stacktrace-" + timestamp + "'>" + new Date() + ": " + contentJson.StackTrace + "</pre>";
            toastr.error(html, contentJson.Message,
                { closeButton: true, timeOut: 0, extendedTimeOut: 0 }
            );
        }
    };


    _this.handleLabelResponse = function (contentJson) {
        _this.Labels = contentJson.Labels;
        _this.Labels.sort(function (a, b) { return a.toLowerCase().localeCompare(b.toLowerCase()); });

        _this.Groups = [];
        var lastGroup = null;
        for (var i = 0; i < _this.Labels.length; i++) {
            var label = _this.Labels[i];
            var underscorePos = label.indexOf('_');
            if (-1 != underscorePos) {
                var group = label.substring(0, underscorePos);
                if (lastGroup !== group) {
                    lastGroup = group;
                    _this.Groups.push(group);
                }
            }
        }
    };


    JnrWebsocket.connect();
    JnrWebsocket.initPing(5004);
});
